@@ -57,7 +57,6 @@
 #include "nsCOMPtr.h"
 #include "nsEscape.h"
 #include "nsJSUtils.h"
-#include "nsIScriptSecurityManager.h"
 #include "nsIDOMWindow.h"
 #include "nsIDOMDocument.h"
 #include "nsIDocument.h"
@@ -82,12 +81,17 @@ GetContextFromStack(nsIJSContextStack *aStack, JSContext **aContext)
 
   nsresult rv = iterator->Reset(aStack);
   NS_ENSURE_SUCCESS(rv, rv);
-  
+
   PRBool done;
   while (NS_SUCCEEDED(iterator->Done(&done)) && !done) {
     rv = iterator->Prev(aContext);
     NS_ASSERTION(NS_SUCCEEDED(rv), "Broken iterator implementation");
 
+    // Consider a null context the end of the line.
+    if (!*aContext) {
+      break;
+    }
+
     if (nsJSUtils::GetDynamicScriptContext(*aContext)) {
       return NS_OK;
     }
@@ -219,8 +223,7 @@ nsLocation::CheckURL(nsIURI* aURI, nsIDocShellLoadInfo** aLoadInfo)
         !principal)
       return NS_ERROR_FAILURE;
     owner = do_QueryInterface(principal);
-
-    GetSourceURL(cx, principal, getter_AddRefs(sourceURI));
+    principal->GetURI(getter_AddRefs(sourceURI));
   }
 
   // Create load info
@@ -423,18 +426,17 @@ NS_IMETHODIMP
 nsLocation::SetHash(const nsAString& aHash)
 {
   nsCOMPtr<nsIURI> uri;
-  nsresult result = NS_OK;
-
-  result = GetWritableURI(getter_AddRefs(uri));
+  nsresult rv = GetWritableURI(getter_AddRefs(uri));
 
   nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
-
   if (url) {
-    url->SetRef(NS_ConvertUCS2toUTF8(aHash));
-    SetURI(url);
+    rv = url->SetRef(NS_ConvertUTF16toUTF8(aHash));
+    if (NS_SUCCEEDED(rv)) {
+      SetURI(url);
+    }
   }
 
-  return result;
+  return rv;
 }
 
 NS_IMETHODIMP
@@ -464,16 +466,16 @@ NS_IMETHODIMP
 nsLocation::SetHost(const nsAString& aHost)
 {
   nsCOMPtr<nsIURI> uri;
-  nsresult result;
-
-  result = GetWritableURI(getter_AddRefs(uri));
+  nsresult rv = GetWritableURI(getter_AddRefs(uri));
 
   if (uri) {
-    uri->SetHostPort(NS_ConvertUCS2toUTF8(aHost));
-    SetURI(uri);
+    rv = uri->SetHostPort(NS_ConvertUTF16toUTF8(aHost));
+    if (NS_SUCCEEDED(rv)) {
+      SetURI(uri);
+    }
   }
 
-  return result;
+  return rv;
 }
 
 NS_IMETHODIMP
@@ -503,16 +505,16 @@ NS_IMETHODIMP
 nsLocation::SetHostname(const nsAString& aHostname)
 {
   nsCOMPtr<nsIURI> uri;
-  nsresult result;
-
-  result = GetWritableURI(getter_AddRefs(uri));
+  nsresult rv = GetWritableURI(getter_AddRefs(uri));
 
   if (uri) {
-    uri->SetHost(NS_ConvertUCS2toUTF8(aHostname));
-    SetURI(uri);
+    rv = uri->SetHost(NS_ConvertUTF16toUTF8(aHostname));
+    if (NS_SUCCEEDED(rv)) {
+      SetURI(uri);
+    }
   }
 
-  return result;
+  return rv;
 }
 
 NS_IMETHODIMP
@@ -682,16 +684,16 @@ NS_IMETHODIMP
 nsLocation::SetPathname(const nsAString& aPathname)
 {
   nsCOMPtr<nsIURI> uri;
-  nsresult result = NS_OK;
-
-  result = GetWritableURI(getter_AddRefs(uri));
+  nsresult rv = GetWritableURI(getter_AddRefs(uri));
 
   if (uri) {
-    uri->SetPath(NS_ConvertUCS2toUTF8(aPathname));
-    SetURI(uri);
+    rv = uri->SetPath(NS_ConvertUTF16toUTF8(aPathname));
+    if (NS_SUCCEEDED(rv)) {
+      SetURI(uri);
+    }
   }
 
-  return result;
+  return rv;
 }
 
 NS_IMETHODIMP
@@ -725,13 +727,11 @@ NS_IMETHODIMP
 nsLocation::SetPort(const nsAString& aPort)
 {
   nsCOMPtr<nsIURI> uri;
-  nsresult result = NS_OK;
-
-  result = GetWritableURI(getter_AddRefs(uri));
+  nsresult rv = GetWritableURI(getter_AddRefs(uri));
 
   if (uri) {
     // perhaps use nsReadingIterators at some point?
-    NS_ConvertUCS2toUTF8 portStr(aPort);
+    NS_ConvertUTF16toUTF8 portStr(aPort);
     const char *buf = portStr.get();
     PRInt32 port = -1;
 
@@ -744,11 +744,13 @@ nsLocation::SetPort(const nsAString& aPort)
       }
     }
 
-    uri->SetPort(port);
-    SetURI(uri);
+    rv = uri->SetPort(port);
+    if (NS_SUCCEEDED(rv)) {
+      SetURI(uri);
+    }
   }
 
-  return result;
+  return rv;
 }
 
 NS_IMETHODIMP
@@ -779,16 +781,16 @@ NS_IMETHODIMP
 nsLocation::SetProtocol(const nsAString& aProtocol)
 {
   nsCOMPtr<nsIURI> uri;
-  nsresult result = NS_OK;
-
-  result = GetWritableURI(getter_AddRefs(uri));
+  nsresult rv = GetWritableURI(getter_AddRefs(uri));
 
   if (uri) {
-    uri->SetScheme(NS_ConvertUCS2toUTF8(aProtocol));
-    SetURI(uri);
+    rv = uri->SetScheme(NS_ConvertUTF16toUTF8(aProtocol));
+    if (NS_SUCCEEDED(rv)) {
+      SetURI(uri);
+    }
   }
 
-  return result;
+  return rv;
 }
 
 NS_IMETHODIMP
@@ -821,17 +823,17 @@ NS_IMETHODIMP
 nsLocation::SetSearch(const nsAString& aSearch)
 {
   nsCOMPtr<nsIURI> uri;
-  nsresult result = NS_OK;
-
-  result = GetWritableURI(getter_AddRefs(uri));
+  nsresult rv = GetWritableURI(getter_AddRefs(uri));
 
   nsCOMPtr<nsIURL> url(do_QueryInterface(uri));
   if (url) {
-    result = url->SetQuery(NS_ConvertUCS2toUTF8(aSearch));
-    SetURI(uri);
+    rv = url->SetQuery(NS_ConvertUTF16toUTF8(aSearch));
+    if (NS_SUCCEEDED(rv)) {
+      SetURI(uri);
+    }
   }
 
-  return result;
+  return rv;
 }
 
 NS_IMETHODIMP
@@ -865,9 +867,9 @@ nsLocation::Reload(PRBool aForceget)
 NS_IMETHODIMP
 nsLocation::Reload()
 {
-  nsCOMPtr<nsIXPCNativeCallContext> ncc;
+  nsAXPCNativeCallContext *ncc = nsnull;
   nsresult rv = nsContentUtils::XPConnect()->
-    GetCurrentNativeCallContext(getter_AddRefs(ncc));
+    GetCurrentNativeCallContext(&ncc);
   NS_ENSURE_SUCCESS(rv, rv);
 
   if (!ncc)
@@ -888,9 +890,9 @@ nsLocation::Reload()
 
     nsIPresShell *shell;
     nsPresContext *pcx;
-    if (doc && (shell = doc->GetShellAt(0)) &&
+    if (doc && (shell = doc->GetPrimaryShell()) &&
         (pcx = shell->GetPresContext())) {
-      pcx->ClearStyleDataAndReflow();
+      pcx->RebuildAllStyleData(NS_STYLE_HINT_REFLOW);
     }
 
     return NS_OK;
@@ -913,6 +915,7 @@ nsLocation::Reload()
     rv = ncc->GetJSContext(&cx);
     NS_ENSURE_SUCCESS(rv, rv);
 
+    JSAutoRequest ar(cx);
     JS_ValueToBoolean(cx, argv[0], &force_get);
   }
 
@@ -1024,27 +1027,3 @@ nsLocation::GetSourceBaseURL(JSContext* cx, nsIURI** sourceURL)
 
   return rv;
 }
-
-nsresult
-nsLocation::GetSourceURL(JSContext* cx, nsIPrincipal* callerPrincipal,
-                         nsIURI** sourceURL)
-{
-  NS_PRECONDITION(callerPrincipal, "Must have caller principal here");
-  
-  *sourceURL = nsnull;
-  
-  nsCOMPtr<nsIDocument> doc;
-  nsresult rv = GetSourceDocument(cx, getter_AddRefs(doc));
-  if (doc) {
-    nsIPrincipal* docPrincipal = doc->GetPrincipal();
-    if (docPrincipal) {
-      PRBool subsumes;
-      rv = callerPrincipal->Subsumes(docPrincipal, &subsumes);
-      if (NS_SUCCEEDED(rv) && subsumes) {
-        NS_IF_ADDREF(*sourceURL = doc->GetDocumentURI());
-      }
-    }
-  }
-
-  return rv;
-}
