@@ -68,9 +68,9 @@ struct EnumerateData {
 };
 
 struct PrefCallbackData {
-  nsIPrefBranch *pBranch;
-  nsISupports   *pObserver;
-  PRBool        bIsWeakRef;
+  nsPrefBranch     *pBranch;
+  nsIObserver      *pObserver;
+  nsIWeakReference *pWeakRef;
 };
 
 
@@ -262,7 +262,7 @@ NS_IMETHODIMP nsPrefBranch::GetComplexValue(const char *aPrefName, const nsIID &
       } else {
         rv = GetCharPref(aPrefName, getter_Copies(utf8String));
         if (NS_SUCCEEDED(rv)) {
-          rv = theString->SetData(NS_ConvertUTF8toUCS2(utf8String).get());
+          rv = theString->SetData(NS_ConvertUTF8toUTF16(utf8String).get());
         }
       }
       if (NS_SUCCEEDED(rv)) {
@@ -332,7 +332,7 @@ NS_IMETHODIMP nsPrefBranch::GetComplexValue(const char *aPrefName, const nsIID &
       return rv;
 
     *_retval = relativePref;
-    NS_ADDREF(NS_STATIC_CAST(nsIRelativeFilePref*, *_retval));
+    NS_ADDREF(static_cast<nsIRelativeFilePref*>(*_retval));
     return NS_OK;
   }
 
@@ -340,7 +340,7 @@ NS_IMETHODIMP nsPrefBranch::GetComplexValue(const char *aPrefName, const nsIID &
     nsCOMPtr<nsISupportsString> theString(do_CreateInstance(NS_SUPPORTS_STRING_CONTRACTID, &rv));
 
     if (NS_SUCCEEDED(rv)) {
-      rv = theString->SetData(NS_ConvertUTF8toUCS2(utf8String));
+      rv = theString->SetData(NS_ConvertUTF8toUTF16(utf8String));
       if (NS_SUCCEEDED(rv)) {
         nsISupportsString *temp = theString;
 
@@ -385,6 +385,8 @@ NS_IMETHODIMP nsPrefBranch::SetComplexValue(const char *aPrefName, const nsIID &
 
   if (aType.Equals(NS_GET_IID(nsILocalFile))) {
     nsCOMPtr<nsILocalFile> file = do_QueryInterface(aValue);
+    if (!file)
+      return NS_NOINTERFACE;
     nsCAutoString descriptorString;
 
     rv = file->GetPersistentDescriptor(descriptorString);
@@ -402,7 +404,7 @@ NS_IMETHODIMP nsPrefBranch::SetComplexValue(const char *aPrefName, const nsIID &
     nsCOMPtr<nsILocalFile> file;
     relFilePref->GetFile(getter_AddRefs(file));
     if (!file)
-      return NS_ERROR_FAILURE;
+      return NS_NOINTERFACE;
     nsCAutoString relativeToKey;
     (void) relFilePref->GetRelativeToKey(relativeToKey);
 
@@ -435,7 +437,7 @@ NS_IMETHODIMP nsPrefBranch::SetComplexValue(const char *aPrefName, const nsIID &
 
       rv = theString->GetData(wideString);
       if (NS_SUCCEEDED(rv)) {
-        rv = SetCharPref(aPrefName, NS_ConvertUCS2toUTF8(wideString).get());
+        rv = SetCharPref(aPrefName, NS_ConvertUTF16toUTF8(wideString).get());
       }
     }
     return rv;
@@ -449,7 +451,7 @@ NS_IMETHODIMP nsPrefBranch::SetComplexValue(const char *aPrefName, const nsIID &
 
       rv = theString->GetData(getter_Copies(wideString));
       if (NS_SUCCEEDED(rv)) {
-        rv = SetCharPref(aPrefName, NS_ConvertUCS2toUTF8(wideString).get());
+        rv = SetCharPref(aPrefName, NS_ConvertUTF16toUTF8(wideString).get());
       }
     }
     return rv;
@@ -459,6 +461,8 @@ NS_IMETHODIMP nsPrefBranch::SetComplexValue(const char *aPrefName, const nsIID &
   // This is deprecated and you should not be using it
   if (aType.Equals(NS_GET_IID(nsIFileSpec))) {
     nsCOMPtr<nsIFileSpec> file = do_QueryInterface(aValue);
+    if (!file)
+      return NS_NOINTERFACE;
     nsXPIDLCString descriptorString;
 
     rv = file->GetPersistentDescriptorString(getter_Copies(descriptorString));
@@ -634,11 +638,10 @@ NS_IMETHODIMP nsPrefBranch::AddObserver(const char *aDomain, nsIObserver *aObser
   if (nsnull == pCallback)
     return NS_ERROR_OUT_OF_MEMORY;
 
-  pCallback->pBranch = NS_STATIC_CAST(nsIPrefBranch *, this);
-  pCallback->bIsWeakRef = aHoldWeak;
+  pCallback->pBranch = this;
+  pCallback->pObserver = aObserver;
 
   // hold a weak reference to the observer if so requested
-  nsCOMPtr<nsISupports> observerRef;
   if (aHoldWeak) {
     nsCOMPtr<nsISupportsWeakReference> weakRefFactory = do_QueryInterface(aObserver);
     if (!weakRefFactory) {
@@ -647,12 +650,11 @@ NS_IMETHODIMP nsPrefBranch::AddObserver(const char *aDomain, nsIObserver *aObser
       return NS_ERROR_INVALID_ARG;
     }
     nsCOMPtr<nsIWeakReference> tmp = do_GetWeakReference(weakRefFactory);
-    observerRef = tmp;
+    NS_ADDREF(pCallback->pWeakRef = tmp);
   } else {
-    observerRef = aObserver;
+    pCallback->pWeakRef = nsnull;
+    NS_ADDREF(pCallback->pObserver);
   }
-  pCallback->pObserver = observerRef;
-  NS_ADDREF(pCallback->pObserver);
 
   mObservers->AppendElement(pCallback);
   mObserverDomains.AppendCString(nsCString(aDomain));
@@ -686,18 +688,7 @@ NS_IMETHODIMP nsPrefBranch::RemoveObserver(const char *aDomain, nsIObserver *aOb
   for (i = 0; i < count; i++) {
     pCallback = (PrefCallbackData *)mObservers->ElementAt(i);
     if (pCallback) {
-     nsCOMPtr<nsISupports> observerRef;
-     if (pCallback->bIsWeakRef) {
-       nsCOMPtr<nsISupportsWeakReference> weakRefFactory = do_QueryInterface(aObserver);
-       if (weakRefFactory) {
-         nsCOMPtr<nsIWeakReference> tmp = do_GetWeakReference(aObserver);
-         observerRef = tmp;
-       }
-     }
-     if (!observerRef)
-       observerRef = aObserver;
-
-      if (pCallback->pObserver == observerRef) {
+      if (pCallback->pObserver == aObserver) {
         mObserverDomains.CStringAt(i, domain);
         if (domain.Equals(aDomain)) {
           // We must pass a fully qualified preference name to remove the callback
@@ -708,7 +699,11 @@ NS_IMETHODIMP nsPrefBranch::RemoveObserver(const char *aDomain, nsIObserver *aOb
             // what we're trying to remove ourselves right now.
             mObservers->RemoveElementAt(i);
             mObserverDomains.RemoveCStringAt(i);
-            NS_RELEASE(pCallback->pObserver);
+            if (pCallback->pWeakRef) {
+              NS_RELEASE(pCallback->pWeakRef);
+            } else {
+              NS_RELEASE(pCallback->pObserver);
+            }
             nsMemory::Free(pCallback);
           }
           return rv;
@@ -732,31 +727,27 @@ NS_IMETHODIMP nsPrefBranch::Observe(nsISupports *aSubject, const char *aTopic, c
 PR_STATIC_CALLBACK(nsresult) NotifyObserver(const char *newpref, void *data)
 {
   PrefCallbackData *pData = (PrefCallbackData *)data;
-  nsPrefBranch *prefBranch = NS_STATIC_CAST(nsPrefBranch *, pData->pBranch);
 
   // remove any root this string may contain so as to not confuse the observer
   // by passing them something other than what they passed us as a topic
-  PRUint32 len = prefBranch->GetRootLength();
+  PRUint32 len = pData->pBranch->GetRootLength();
   nsCAutoString suffix(newpref + len);  
 
   nsCOMPtr<nsIObserver> observer;
-  if (pData->bIsWeakRef) {
-    nsIWeakReference *weakRef = NS_STATIC_CAST(nsIWeakReference *, pData->pObserver);
-    observer = do_QueryReferent(weakRef);
+  if (pData->pWeakRef) {
+    observer = do_QueryReferent(pData->pWeakRef);
     if (!observer) {
       // this weak referenced observer went away, remove them from the list
-      nsCOMPtr<nsIPrefBranch2> pbi = do_QueryInterface(pData->pBranch);
-      if (pbi) {
-        observer = NS_STATIC_CAST(nsIObserver *, pData->pObserver);
-        pbi->RemoveObserver(newpref, observer);
-      }
+      pData->pBranch->RemoveObserver(newpref, pData->pObserver);
       return NS_OK;
     }
-  } else
-    observer = NS_STATIC_CAST(nsIObserver *, pData->pObserver);
+  } else {
+    observer = pData->pObserver;
+  }
 
-  observer->Observe(pData->pBranch, NS_PREFBRANCH_PREFCHANGE_TOPIC_ID,
-                    NS_ConvertASCIItoUCS2(suffix).get());
+  observer->Observe(static_cast<nsIPrefBranch *>(pData->pBranch),
+                    NS_PREFBRANCH_PREFCHANGE_TOPIC_ID,
+                    NS_ConvertASCIItoUTF16(suffix).get());
   return NS_OK;
 }
 
@@ -784,7 +775,11 @@ void nsPrefBranch::freeObserverList(void)
           // what we're trying to remove right now.
           mObservers->ReplaceElementAt(nsnull, i);
           PREF_UnregisterCallback(pref, NotifyObserver, pCallback);
-          NS_RELEASE(pCallback->pObserver);
+          if (pCallback->pWeakRef) {
+            NS_RELEASE(pCallback->pWeakRef);
+          } else {
+            NS_RELEASE(pCallback->pObserver);
+          }
           nsMemory::Free(pCallback);
         }
       }
@@ -874,8 +869,8 @@ PR_STATIC_CALLBACK(PLDHashOperator)
 pref_enumChild(PLDHashTable *table, PLDHashEntryHdr *heh,
                PRUint32 i, void *arg)
 {
-  PrefHashEntry *he = NS_STATIC_CAST(PrefHashEntry*, heh);
-  EnumerateData *d = NS_REINTERPRET_CAST(EnumerateData *, arg);
+  PrefHashEntry *he = static_cast<PrefHashEntry*>(heh);
+  EnumerateData *d = reinterpret_cast<EnumerateData *>(arg);
   if (PL_strncmp(he->key, d->parent, PL_strlen(d->parent)) == 0) {
     d->pref_list->AppendElement((void*)he->key);
   }
